#!/usr/bin/perl

use strict;

push @INC,"./scripts";

require("common.pl");

my @nets = &nets();
my %cs = &nets(1);
my $interp_macro="print \">\"; while(<STDIN>) { \$!=\"\"; print(eval \$_,\"\n\$!\n>\");}";
my $float=qr/-?(\d+\.?\d*|\.\d+)([e|E]-?\d+)?/;

my %render_script = (
	'fig1' => 'render_fig1.pl',
	'fig2' => 'render_fig2.R',
	'fig3' => 'render_fig3.R',
	'fig4' => 'render_fig4.R',
	'fig5' => 'render_fig5.R',
	'table' => 'render_table.R');


sub clear()
{
	print "\e[2J";
}

sub main()
{
	my $task;
	my @action_list = ('1','2','3','4','5','t','q');
	my %actions = (
		'1' => [ "Figure 1: Toronto 20 routed on 2 architectures", 'fig1',\&fig1 ],
		'2' => [ "Figure 2: Distribution of permutations of alu4", 'fig2',\&fig2 ],
		'3' => [ "Figure 3: Routed vs. feasible delays for changing channel width",
			'fig3',\&fig3 ],
		'4' => [ "Figure 4: Routed Delay vs. Specified Delay Target", 'fig4', \&fig4 ],
		
		# Note: figure 3 is the summary of figure 5 and generated from the same data
		'5' => [ "Figure 5: Channel width sweep comparing different routing parameters",
			'fig3', \&fig3 ],
		't' => [ "Table 1:  Comparison of delay distribution for different routing\n\t".
		          "  parameters for all benchmarks" ,
			  'table',\&table ],
	  	'q' => [ "quit" ]
	);

	my $prompt = "Please select a task [1-5,t,q] > ";
	while(1)
	{
		clear;
		print "Select task:\n\n\n";
		foreach my $act (@action_list)
		{
			print $actions{$act}[0]."\n\n";
		}
		print $prompt;

		$task = lc(<STDIN>);
		chomp $task;

		if(exists $actions{$task})
		{
			if($task eq "q")
			{
				exit(0);
			}

			&run_target($actions{$task});
		} else {
			$prompt = "bad command \"$task\", press enter to try again\n".
			"Please select a task [1-5,t,q] > "
		}
	}
}

sub select_some($@)
{
	my $title = shift;
	my $prompt = shift;
	my $default = shift;
	my @item_list = @_;
	my %items = map { $_ => $default } @item_list;

	my $item = " ";

	while($item ne "")
	{
		clear;
		print "$title\n\n";
		for(my $i = 0; $i < scalar @item_list; $i++)
		{
			printf("%2d ) [%s] $item_list[$i]\n",$i,($items{$item_list[$i]}?"X":" "));
		}
		print "\n";
		printf("%2d ) All on\n",scalar @item_list);
		printf("%2d ) Toggle all\n",1+scalar @item_list);
		printf("%2d ) All off\n",2+scalar @item_list);

		print("\n$prompt\n[0-".(2 + scalar @item_list)."] > ");
		$item = <STDIN>;
		chomp $item;

		if($item=~/^\d+$/)
		{
			# str to int
			$item = eval($item);
			if($item < scalar @item_list)
			{
				$item = $item_list[$item];
				$items{$item} = not $items{$item};
			} elsif($item == scalar @item_list) {
				@items{@item_list} = ((1) x (scalar @item_list) );
			} elsif($item == 1 + scalar @item_list) {
				@items{@item_list} = map { not $_ } @items{@item_list};
			} elsif($item == 2 + scalar @item_list) {
				@items{@item_list} = ((0) x (scalar @item_list) );
			}
		}
	}
	return grep { $items{$_} } @item_list;
}

sub select_one($@)
{
	my $title = shift;
	my $prompt = shift;
	my $default = shift;
	my @item_list = @_;
	my %items = map { $_ => 0 } @item_list;

	if(length($default) and exists($items{$default}))
	{
		$items{$default} = 1;
	}

	my $item = " ";

	while($item ne "")
	{
		clear;
		print "$title\n\n";
		for(my $i = 0; $i < scalar @item_list; $i++)
		{
			printf("%2d ) [%s] $item_list[$i]\n",$i,($items{$item_list[$i]}?"X":" "));
		}
		print "\n";
		printf("%2d ) None\n",scalar @item_list);

		print("\n$prompt\n[0-".(scalar @item_list)."] > ");
		$item = <STDIN>;
		chomp $item;

		if($item=~/^\d+$/)
		{
			# str to int
			$item = eval($item);
			if($item < scalar @item_list)
			{
				@items{@item_list} = ((0) x (scalar @item_list) );
				$item = $item_list[$item];
				$items{$item} = not $items{$item};
			} elsif($item == scalar @item_list) {
				@items{@item_list} = ((0) x (scalar @item_list) );
			}
		}
	}
	return grep { $items{$_} } @item_list;
}

sub route_base($)
{
	my $title = shift;
	my $default = shift;
	my @dstrs = ("yn","Yn","yN");
	my @dval = ("","y","n");
	my $ans = "";

	if(! defined $default) { $default = 0; }

	while(! length($ans))
	{
		clear;
		print("$title\n\n");
		print("Route the original unperturbed netlist?\n\n");
		print("[$dstrs[$default]] > ");

		$ans = lc(<STDIN>);
		chomp $ans;

		if($ans =~ /^\s*(y|n)?$/i)
		{
			$ans = $1;
			if(! defined($ans)) { $ans = $dval[$default]; }
		}
		else
		{
			$ans = "";
		}
	}

	if($ans eq "y")
	{
		return "--base";
	}
	else
	{
		return undef;
	}
}

sub get_number_list($$@)
{
	my $title = shift;
	my $what = shift;

	my @res = @_;

	while(1)
	{
		clear;
		print "$title\n\n";
		print "Enter a list of $what\n";
		print "Use spaces or commas to separate individual values and ragnes\n";
		print "Specify ranges as min..max or min-max\n";
		print "\nCurrent items:\n";
		map { print("$_\n"); } @res;
		print "\nEnter 'c' to clear the list\n";
		print "\nPress enter with no values to continue\n";
		print("> ");
		my $item = <STDIN>;
		chomp $item;

		if($item=~/^\s*\d+((\.\.|-|,|(\s+))\d+)*\s*$/) {

			$item =~ s/(^\s+)|(\s+$)//g;
			$item =~ s/\s+/ /g;

			push @res,$item;

		} elsif($item =~ /^\s*c\s*$/i) {
			@res = ();
		}elsif($item =~ /^\s*$/) {
			return @res;
		}
	}
}

sub get_float_list($$@)
{
	my $title = shift;
	my $what = shift;

	my @res = @_;

	while(1)
	{
		&clear;
		print "$title\n\n";
		print "Enter a list of $what\n";
		print "Use spaces or commas to separate values\n";
		print "\nCurrent items:\n";
		map { print("$_\n"); } @res;
		print "\nEnter 'c' to clear the list\n";
		print "\nPress enter with no values to continue\n";
		print("> ");
		my $item = <STDIN>;
		chomp $item;

		if($item=~/^\s*($float)\s*$/) {

			push @res,$1;

		} elsif($item =~ /^\s*c\s*$/i) {
			@res = ();
		}elsif($item =~ /^\s*$/) {
			return @res;
		}
	}
}


sub run_target(\@)
{
	my ($title,$target,$route_function) = @{$_[0]};
	my %actions = map { $_ => 1 } &select_some($title,
		"Please select actions to perform",
		1,"Route", "Harvest", "Render");

	if(exists $actions{'Route'})
	{
		&clear;
		&{$route_function}($title);
		print "\npress enter to continue\n";
		<STDIN>;
	}

	if(exists $actions{'Harvest'})
	{
		&clear;
		print("Harvesting route results for \"$title\"\n");
		my $command = ("./scripts/harvest_${target}.pl");
		print("running: $command\n");
		system($command);
		print "\npress enter to continue\n";
		<STDIN>;
	}

	if(exists $actions{'Render'})
	{
		&clear;
		print("Rendering route results for \"$title\"\n");
		my $command = ("./scripts/".($render_script{$target}));
		print "fig 1 script",$render_script{'fig1'}."\n";
		print("running: $command\n");
		system($command);
		print "\npress enter to continue\n";
		<STDIN>;
	}

	&clear;
	print "Finished processing \"$title\"\n\npress enter to return to the main menu\n";
	<STDIN>;
	#read ret;
}

sub perturb($$)
{
	my @nets = @{shift @_};
	my $range = shift;

	if(not scalar @nets)
	{
		return;
	}

	my @command=("./scripts/shuffle_netlist.pl","-q",join(",",@nets),$range);

	&clear;
	print "Checking netlist pertubations to prepare for routing\n";
	print "\nRunning: ";
	print join(" ",@command);
	print "\n";
	print "\nPress enter to continue\n";
	<STDIN>;
	system(@command);
}

my $rcount = 0;
sub route($)
{
	my $title = shift;
	my @command = @_;
	&clear;
	print "$title\n\nReady press enter to run this command:\n";
	print(join(" ",@command));
	print("\n");
	<STDIN>;
	system(@command);
}

sub fig1($)
{
	my $title = shift;
	my @act_nets = &select_some($title,
		"Please select nets to route:",0,@nets);
	my @command = ("./scripts/fig1.sh");
	if((scalar @nets) == (scalar @act_nets))
	{
		push @command,"--all";
	}
	elsif(scalar @act_nets)
	{
		push @command,@act_nets;
	}

	if((scalar @command) > 1)
	{
		&route($title,@command);
	}
	else
	{
		print "No nets selected.\n";
	}
}

sub fig2($)
{
	my $title = shift;
	my @act_nets = &select_one($title,
		"Please select a net to route:","alu4",@nets);
	my @command = ("./scripts/fig2.sh",@act_nets);

	if(scalar(@command) != 2)
	{
		print "No nets selected.\n";
		return;
	}

	my $base = &route_base($title,1);
	my $number_list = join(",",&get_number_list($title,"pertubations to route"));

	if(defined($base))
	{
		push(@command,$base);
	}

	if(length($number_list))
	{
		&perturb(\@act_nets,$number_list);
		push(@command,$number_list);
	}

	if(scalar @command == 2)
	{
		print "Nothing to be requested, not running the route script\n";
		return;
	}

	&route($title,@command);
}

sub fig3($)
{
	my $title = shift;
	my @act_nets = &select_one($title,
		"Please select a net to route:","alu4",@nets);
	my @command = ("./scripts/fig3.sh",@act_nets);

	if(scalar(@command) != 2)
	{
		print "No nets selected.\n";
		return;
	}

	my @pfms = &select_some($title,
		"Please select presure factor multipliers to run (tcs for Delay Target search):",
		1,"1.1","1.3","tcs");

	if(! scalar(@pfms))
	{
		print "No pfms selected.\n";
		return;
	}
	
	push @command,join(",",@pfms);


	my $chans = join(",",&get_number_list($title,"channel widths",$cs{$act_nets[0]}."-".($cs{$act_nets[0]}+10)));

	if(! length($chans))
	{
		print "No channel widths selected.\n";
		return;
	}
	push @command,join(",",$chans);

	my $base = &route_base($title,1);
	if(defined($base))
	{
		push(@command,$base);
	}

	my $ns = join(",",&get_number_list($title,"pertubations to route"),"1-10");
	if(length($ns))
	{
		&perturb(\@act_nets,$ns);
		push(@command,$ns);
	}else
	{
		if(not defined $base)
		{
			print("No versions of the netlist selected\n");
			return;
		}
	}

	&route($title,@command);
}

sub fig4($)
{
	my $title = shift;

	my @act_nets = &select_one($title,
		"Please select a net to route:","alu4",@nets);
	my @command = ("./scripts/fig4.sh",@act_nets);

	if(scalar(@command) != 2)
	{
		print "No nets selected, not routing.\n";
		return;
	}

	my @delays = &get_float_list($title,"target delays (in seconds)");
	if(not scalar @delays)
	{
		print "No delays selected, not routing.\n";
		return;
	}

	push @command,join(",",@delays);

	my $ns = join(",",&get_number_list($title,"pertubations to route"),"1-10");
	if(length($ns))
	{
		&perturb(\@act_nets,$ns);
		push(@command,$ns);
	}

	&route($title,@command);
}

sub table($)
{
	my $title = shift;
	my @act_nets = &select_some($title,
		"Please select nets to route:",0,@nets);
	my @command = ("./scripts/table.sh");
	if((scalar @nets) == (scalar @act_nets))
	{
		push @command,"--all";
	}
	elsif(scalar @act_nets)
	{
		push @command,join(",",@act_nets);
	}
	else
	{
		print "No nets selected\n";
		return;
	}

	my $ns = join(",",&get_number_list($title,"pertubations to route","1-10"));
	if(length($ns))
	{
		&perturb(\@act_nets,$ns);
		push(@command,$ns);
	}

	&route($title,@command);
}

&main()

