#!/bin/sh

base=`/bin/pwd`

. $base/scripts/util.sh

run_one()
{
	if [ -r ${base}/inputs/net_shuffled/${net}/$n.net ]
	then
		if [ ! -e ${n}.log ]
		then
			${base}/src/VPR_HET/vpr \
				${base}/inputs/net_shuffled/${net}/$n.net \
				${base}/inputs/arch/k4-n4_nom.xml \
				${base}/inputs/place/${net}.place \
				${net}.route \
				-nodisp -route_only \
				-route_chan_width $c \
				$(pfm_flags $pfm) \
				| tee ${n}.log
		fi
	fi
}

run_base()
{
	if [ ! -e base.log ]
	then
		${base}/src/VPR_HET/vpr \
			${base}/inputs/net/${net}.net \
			${base}/inputs/arch/k4-n4_nom.xml \
			${base}/inputs/place/${net}.place \
			${net}.route \
			-nodisp -route_only \
			-route_chan_width $c \
			$(pfm_flags $pfm) \
			| tee base.log
	fi
}



usage()
{
	echo "Usage: fig3.sh <net name> <pfm list> <channel widths> [--base] <list of targets>"
	echo ""
	echo "Note: only even channel widths will be run"
	number_list_usage
	echo ""
	echo "Example: ./scripts/fig3.sh alu4 1.3,1.1,tcs 28-32 --base 1-10"
	exit	
}

if [ $# = 0 -o "x$1" = "x--help" ]
then
	usage
fi

net=$1
check_net $net
shift

pfms=$(echo $1 | sed 's/,/ /g')
shift

cs=$(parse_cs_list $1)
shift

bases=0
if [ "x$1" = "x--base" ]
then
	bases=1
	echo bases on
	shift
fi

targets=
if [ $# -gt 0 ]
then
	targets=$(number_list $1)
	shift
fi

for c in $cs
do
	for pfm in $pfms
	do
		if [ $bases -gt 0 ]
		then
			mkdir -p ${base}/experiments/fig3/${net}/${c}/${pfm}
			cd ${base}/experiments/fig3/${net}/${c}/${pfm}
			run_base
		fi
		for n in $targets
		do
			mkdir -p ${base}/experiments/fig3/${net}/${c}/${pfm}/$n
			cd ${base}/experiments/fig3/${net}/${c}/${pfm}/$n
			run_one
		done
	done
done
