#!/bin/sh

base=`/bin/pwd`

. $base/scripts/util.sh

run_one()
{
	if [ -r ${base}/inputs/net_shuffled/${net}/$n.net ]
	then
		if [ ! -e ${n}.log ]
		then
			${base}/src/VPR_HET/vpr \
				${base}/inputs/net_shuffled/${net}/$n.net \
				${base}/inputs/arch/k4-n4_nom.xml \
				${base}/inputs/place/${net}.place \
				${net}.route \
				-nodisp -route_only \
				-route_chan_width `grep ${net} ${base}/inputs/cs | cut -f 2 -d =` \
				-target_delay $tdel -force_target_delay \
				-max_criticality 0.999 \
				-pres_fac_mult 1.1 \
				-max_router_iterations 150 \
				| tee ${n}.log
		fi
	fi
}

usage()
{
	echo "Usage: fig4.sh <net name> <target delays> <list of targets>"
	echo ""
	echo "Target delays should be expressed as a comma separated list"
	echo "of delays in seconds."
	echo "example 9.5e-9,10e-9,10.5e-9"
	echo ""
	number_list_usage
	echo ""
	echo "Example:"
	echo "   ./scripts/fig4.sh alu4 9.5e-9,10e-9,10.5e-9 1-10" 
	exit	
}

if [ $# = 0 -o "x$1" = "x--help" ]
then
	usage
fi

net=$1
check_net $net
shift

tdels=$(echo $1 | sed 's/,/ /g')
shift

targets=
if [ $# -gt 0 ]
then
	targets=$(number_list $1)
	shift
fi

for tdel in $tdels
do
	for n in $targets
	do
		mkdir -p ${base}/experiments/fig4/${net}/${tdel}/$n
		cd ${base}/experiments/fig4/${net}/${tdel}/$n
		run_one
	done
done
