#!/usr/bin/Rscript

all_data <- na.omit(read.table("fig3.dat",header=T))

fig3 <- function(dat)
{
	dat=dat[sort(dat$c,index.return=T)$ix,]
	pdf(sprintf("fig3_%s.pdf",dat[1,]$net));
	# identify normal vpr and original netlist records
	vpr = dat[(dat$pfm == "1.3" & dat$n == -1),];
	xlim = c(-0.5, 0.5) + (cw_range = range(dat$c))
	ylim = c(-0.2, 0.5) + (cp_range = range(dat$cp))
	plot(y=vpr$cp, x=vpr$c,
		xlim = xlim, ylim = ylim,
		type='b',pch='+',xaxt='n',yaxt='n',bg='white',
		xlab="Channel Width",
		ylab="Delay (ns)",
		bty='n')
	title(sprintf("Figure 3 for %s",dat[1,]$net));
        grid(ny=NULL,nx=0,col="#000000",lty="43",lwd=0.25)
        lines(y=vpr$cp,x=vpr$c,type='b',pch='+',xaxt='n',yaxt='n')
	axis(1,col='white');
        axis(4,col='white');
	best=t(mapply(function(x){c(c=x[1,]$c,cp=min(x$cp))},split(dat,factor(dat$c))))
        lines(best,type='b',pch='x');
        legend('topright',c("original ordering  ","fastest observed"),horiz=F,pch=c('+','x'),box.col='white')
	dev.off()
}

fig5 <- function(dat)
{
	pdf(sprintf("fig5_%s.pdf",dat[1,]$net));
	xlim = c(-0.5, 0.5) + (cw_range = range(dat$c))
	ylim = c(-0.2, 0.5) + (cp_range = range(dat$cp))
	a = split(dat,factor(dat$pfm))
        ben_colors = c("#ff9999","#eeeeff","#444444");
        ben_borders = c("#801010","#444488","#000000");
        plot.new(); plot.window(xlim=xlim,ylim=ylim)
	title(main=sprintf("Figure 5 for %s",dat[1,]$net),
		xlab="Channel Width",
		ylab="Delay (ns)");
	axis(1)
	axis(2)
	box()
	 boxplot(formula= a[["1.3"]]$cp ~ a[["1.3"]]$c, at=sort(unique(a[["1.3"]]$c))-0.4,boxwex=0.35,names=F,add=T,col=ben_colors[1],border=ben_borders[1],axes=F
        ,lwd=0.75,medlwd=1,outcex=0.75
        )

        boxplot(formula= a[["1.1"]]$cp ~ a[["1.1"]]$c, at=sort(unique(a[["1.1"]]$c)),boxwex=0.35,add=T,col=ben_colors[2],border=ben_borders[2],xlab="Channel Width",ylab="Delay (ns)"
        ,lwd=0.75,medlwd=1,outcex=0.75
        )
        boxplot(formula= a[["tcs"]]$cp ~ a[["tcs"]]$c, at=sort(unique(a[["tcs"]]$c))+0.4,boxwex=0.35,add=T,names=F,col=ben_colors[3],border=ben_borders[3],axes=F
        ,lwd=0.75,medlwd=1,outcex=0.75
)
	legend("topright",c("pfm=1.3","pfm=1.1","Delay Targeted"),border=ben_borders,fill=ben_colors,bty='n');
	dev.off()
}

by(all_data,all_data$net,fig3)
by(all_data,all_data$net,fig5)
