#!/usr/bin/Rscript

all_data <- na.omit(read.table("fig4.dat",header=T))
all_data <- all_data[all_data$cp > 0,]
#all_data$tdel = as.numeric(all_data$tdel)*1e9

fig4 <- function(dat)
{
	pdf(sprintf("fig4_%s.pdf",dat[1,]$net));
	orig_par = par(mar=par()$mar+c(0,0,0,2))
	
	# separate out the base routes
	vpr = dat[dat$n < 0,]
	dat = dat[dat$n >= 0,]

	# assuming that each n shows up at least once and all
        # shuffles were attempted for each tdel.
        trials = length(unique(dat$n))
        yields=data.frame(t(mapply(function(x){c(tdel=x[1,]$tdel,yield=nrow(x))},
		split(dat,factor(dat$tdel)))))
        yields$yield = yields$yield*(100.0/trials)
	target_range = range(dat$tdel)
	xlim = target_range + c(-1,1) * min(diff(sort(unique(dat$tdel))))/1.9
	boxplot(formula = dat$cp ~ dat$tdel
			,at=sort(unique(dat$tdel))
			,boxwex=0.2
			,ylab="Routed Delay (ns)"
			,xlab="Target (ns)"
			,col='gray'
			,lwd=0.75,medlwd=1,outcex=0.75
			,xlim = xlim
			,add=F
	       )

	title(sprintf("Figure 4 for %s",dat[1,]$net));
	
	# adjust the plot y range for yield
	usr = par()$usr;
	usr[3:4]=c(0,104);
	old_usr = par(usr=usr)
	
	# draw the yield curve, label it and add some background guide lines
	axis(4)
	mtext("Yield (%)",4,line=2.5)
	grid(ny=NULL,nx=0,col="#000000",lty="43",lwd=0.25)
	lines(yields,type='b',pch=4)
	
	par(old_usr)
	# diagnal line to highlight the relation between requested
	# and routed delay
	abline(0,1,col="gray",lty=2,lwd=1)
	
	# redraw the boxes to make sure they are on top
	boxplot(formula = dat$cp ~ dat$tdel
			,at=sort(unique(dat$tdel))
			,boxwex=0.2
			,ylab="Routed Delay (ns)"
			,xlab="Target (ns)"
			,col='gray'
			,lwd=0.75,medlwd=1,outcex=0.75
			,xlim = xlim
			,add=T
	       )
	
	
	par(usr=c(0,1,0,1))

	legend(x=0.05,y=0.95,c('yield','delay'),pch=c(4,-1)
		,fill=c(0,'gray')
		,border=c(0,'black')
		,bty='n');

	par(orig_par)

}

by(all_data,all_data$net,fig4)
