#!/usr/bin/Rscript --save

interleave <- function(v1,v2)
{
	c(mapply('c',v1,v2))
}

CDFSparkString <- function(data)
{
	r <- range(data);
	data = (data-r[1])/(r[2]-r[1])
	data = round(10*data) # short cut to limit number of segments
	b = c(by(data,data,length)); # frequency count
	b=b[order(as.numeric(labels(b)))]; # sort just to be sure

	
	# make 2 points for each cp value (start and stop of
	# the segment
	xs=c(mapply(function(x) c(x,x),as.numeric(labels(b))))
	counts = c(mapply(function(x) c(1,x-1),b))
	counts[1:2] = c(0,sum(counts[1:2])); # start the first point at 0
	csum = cumsum(counts); # convert from frequency to cumulative value


	n = length(xs);
	xst = xs - xs[1]; # translate x values to start at 0

	# normalize both x and y to 0:1, interleave and join with spaces
	return(paste(c(mapply(c, xst/xst[n], csum/csum[n])),
		     collapse=" "));
}


generateSparkEnv <- function(data,norm) {
	r <- range(data);
	data = (data-r[1])/(r[2]-r[1])
	norm = (norm - r[1])/(r[2] - r[1])
	data = round(data*1,4)/1 # short cut to limit number of segments
	norm = round(norm*1,4)/1 # short cut to limit number of segments
	b = c(by(data,data,length)); # frequency count
	b=b[order(as.numeric(labels(b)))]; # sort just to be sure

	
	# make 2 points for each cp value (start and stop of
	# the segment
	xs=c(mapply(function(x) c(x,x),as.numeric(labels(b))))
	counts = c(mapply(function(x) c(1,x-1),b))
	counts[1:2] = c(0,sum(counts[1:2])); # start the first point at 0

	keep = (1:length(counts))[sort(unique(counts>0))];
	xs = xs[keep]
	counts=counts[keep]
	csum = cumsum(counts); # convert from frequency to cumulative value

	n = length(xs);

	# normalize both x and y to 0:1, interleave and join with spaces
	sparkLine <- paste(c(mapply(c, xs, csum/csum[n])),
		     	   collapse=" ");

	spEnv <- paste("\\begin{sparkline}{10} \n \\sparkdot",
			norm,
			ecdf(data)(norm),
			" red \n",
			"\\spark ", sparkLine,
			"/ \n",
			"\\spark ", mean(data), 0,
				    mean(data), 1,
			"/ \n\\end{sparkline}",
			collapse=" "
	)
}



ftns <- format_to_n_sig <- function(x,n,m=0) 
{
	if(length(x)==0)
	{
		return(" ");
	}
	return(sprintf(sprintf("%%0.%df",m),(signif(round(x,max(0,n-1)),n))));
}

generateRow <- function(data)
{
	spark = generateSparkEnv(data[data$pfm=='1.3',]$cp,
		norm=data[data$type=='normal'&data$pfm=='1.3',][1,]$cp);

	net = data[1,]$net;
	w = data[1,]$c;

	stats=by(data,data$pfm,function(x) 
	{
		r = range(x$cp);
		c('m'=median(x$cp), 's'=(100*(r[2]-r[1])/r[1]));
	});

	ret <- data.frame(net=sprintf("{\\tt %s}",net), w=w, cdf=spark
			,m13  = ftns(stats[[c("1.3","m")]],2,1)
			,s13  = ftns(stats[[c("1.3","s")]],2)
			,m11  = ftns(stats[[c("1.1","m")]],2,1)
			,s11  = ftns(stats[[c("1.1","s")]],2)
			,mtcs = ftns(stats[[c("tcs","m")]],2,1)
			,stcs = ftns(stats[[c("tcs","s")]],2,1)
			,stringsAsFactors = F
	);
	return(ret);
}

# Simple trick to simplify testing and debugging with an interpreter
go <- function() source('scripts/table.R')

data <- na.omit(read.table("table.dat",header=T))
result = by(data,data$net,generateRow)
result = (t(mapply(as.matrix,result)))
write.table(result,file = "./table_dat.tex",sep = ' & ',eol = '\\\\\n',col.names = F,row.names = F, na = ' ',quote = F);

table_wrapper <- "
\\documentclass{article}

\\usepackage{color}
\\usepackage{sparklines}

\\begin{document}

\\begin{tabular}{|r|c|cr|r|r|r|r|r|}\\hline
	& &\\multicolumn{5}{c|}{VPR 5.0.2}  & \\multicolumn{2}{c|}{Delay} \\\\
	& & \\multicolumn{3}{c|}{$pfm=1.3$} & \\multicolumn{2}{c|}{$pfm=1.1$} & \\multicolumn{2}{c|}{Targeted} \\\\
	Design & W & CDF & med. & \\% & med. & \\% & med. & \\%  \\\\
\\hline\\hline
\\input table_dat.tex
\\hline\\hline
\\end{tabular}

\\end{document}
";
tw_file <- file("table.tex","w");
cat(table_wrapper,file=tw_file);
close(tw_file);

system("pdflatex table.tex");
