#!/usr/bin/perl

use strict;


my $quiet = 0;
if($ARGV[0] eq "-q")
{
	$quiet = 1;
	shift @ARGV;
}
my $net_names = $ARGV[0];
my $range_str = $ARGV[1];
my @range = ();

# check and clean up ranges
if($range_str =~ /^[0-9,\.-]+$/)
{
	$range_str =~ s/-/../g;
	@range = eval($range_str);
	my %hash;
	@hash{@range}=1;
	@range = sort { $a <=> $b } keys %hash;
}

foreach my $net_name (split /,/, $net_names)
{
	mkdir "inputs/net_shuffled" if(! -d "inputs/net_shuffled");
	mkdir "inputs/net_shuffled/$net_name" if(! -d "inputs/net_shuffled/$net_name");

	my @range = @range;

	map
	{
		my $target = sprintf("inputs/net_shuffled/$net_name/%0.5d.net",$_);
		&shuf_net("inputs/net/${net_name}.net",$target);
	} @range;

	sub shuf_net($$)
	{
		if(-e $_[1]) {
			if(not $quiet)
			{
				print("Already have $_[1]\n");
			}
			return;
		}
		print("Generating $_[1]\n");
		open(my $input,"<$_[0]") or die "couldn't read $_[0]";

		my $out_file=$_[1];

		my $head="";
		my @clbs=();
		my $current="";

		while(<$input>)
		{
			if(! length $current)
			{
				if(/^\s*\.clb/)
				{
					$current=$_;
				}
				else
				{
					$head.=$_;
				}
			}
			else
			{
				if(/^\s*\.clb/)
				{
					push @clbs,$current;
					$current=$_;
				}
				else
				{
					$current.=$_;
				}
			}
		}
		if(length $current)
		{
			push @clbs,$current;
		}

		close($input);

		sub shuffle(@)
		{
			my @out = @_;
			my $j;
			my $t;
			my $i=scalar(@out);
			while($i)
			{
				$j=int(rand($i--));
				next if($j == $i);
				$t = $out[$j];
				$out[$j] = $out[$i];
				$out[$i] = $t;
			}
			return @out;
		}

		open(my $out,">$out_file") or die "could not write to $out_file";
		
		print $out $head;
		map { print $out $_} shuffle(@clbs);
		close $out;
	}
}
