#!/bin/sh

base=`/bin/pwd`

. $base/scripts/util.sh

run_one()
{
	if [ -r ${base}/inputs/net_shuffled/${net}/$n.net ]
	then
		if [ ! -e ${n}.log ]
		then
			${base}/src/VPR_HET/vpr \
				${base}/inputs/net_shuffled/${net}/$n.net \
				${base}/inputs/arch/k4-n4_nom.xml \
				${base}/inputs/place/${net}.place \
				${net}.route \
				-nodisp -route_only \
				-route_chan_width `grep ${net} ${base}/inputs/cs | cut -f 2 -d =` \
				$(pfm_flags $pfm) \
				| tee ${n}.log
		fi
	fi
}

run_base()
{
	if [ ! -e base.log ]
	then
		${base}/src/VPR_HET/vpr \
			${base}/inputs/net/${net}.net \
			${base}/inputs/arch/k4-n4_nom.xml \
			${base}/inputs/place/${net}.place \
			${net}.route \
			-nodisp -route_only \
			-route_chan_width `grep ${net} ${base}/inputs/cs | cut -f 2 -d =` \
			$(pfm_flags $pfm) \
			| tee base.log
	fi
}



usage()
{
	echo "Usage: table.sh <list of net names> <list of targets>"
	echo ""
	number_list_usage
	echo ""
	echo "Examples: ./scripts/table.sh alu4,des,tseng 1-10"
	echo "          ./scripts/table.sh --all 1-10"
	exit	
}

if [ $# = 0 -o "x$1" = "x--help" ]
then
	usage
fi

if [ "x$1" = "x--all" ]
then
	nets=`cat ${base}/inputs/cs | cut -f 1 -d =`
else
	nets=$(echo $1 | sed 's/,/ /g')
fi
shift



targets=
if [ $# -gt 0 ]
then
	targets=$(number_list $1)
	shift
fi

# first a sanity check
for net in $nets
do
	check_net $net
done


for net in $nets
do
	for pfm in 1.3 1.1 tcs
	do
		# base
		mkdir -p ${base}/experiments/table/${net}/${pfm}
		cd ${base}/experiments/table/${net}/${pfm}
		run_base

		for n in $targets
		do
			mkdir -p ${base}/experiments/table/${net}/${pfm}/$n
			cd ${base}/experiments/table/${net}/${pfm}/$n
			run_one
		done
	done
done
